@@ -29,6 +29,7 @@ def bacs(l, ict, Sll, Xa, Ma, P, *,
          near_ratio = 1.0,
          sigma_h = (0, 0, 0, 0, 0, 0, 100),
          sigma_c = None,
+         sigma_i = None,
          ):
     """
     Perform a bundle adjustment for multi-view cameras based on
@@ -64,12 +65,13 @@ def bacs(l, ict, Sll, Xa, Ma, P, *,
         near_ratio  fraction of scene points used for datum definition (default: 1 -> all)
         sigma_h  variances of centroid constraints (3 translations, 3 rotations, 1 scale, default: (0,0,0,0,0,0,100))
         sigma_c  variances of camera motion constraints (experimental, length 6xT, default: None)
+        sigma_i  variances of object point constraints (experimental, length 3xI, default: None)
 
     Output:
         la   estimated camera rays [3xN]
         Xa   estimated object point coordinates (homogeneous) [4xN]
         Ma   estimated motion matrices [Tx4x4] (from object system to MCS)
-        Sdd  estimated covariance matrix of oriantation parameters [6Tx6T]
+        Sdd  estimated covariance matrix of orientation parameters [6Tx6T]
         s0dsq  estimated variance factor
         vr   estimated corrections on l in tangent space
         w    estimated weights on diagonal elements of Sll
@@ -81,9 +83,11 @@ def bacs(l, ict, Sll, Xa, Ma, P, *,
     N = l.shape[1]
     I = Xa.shape[1]
     T = len(Ma)
-    c_indices = [k for k, s in enumerate(sigma_c) if s is not None and not np.isinf(s)] if sigma_c is not None else [] 
-    sigma_c = [s for s in sigma_c if s is not None and not np.isinf(s)] if sigma_c is not None else []
-    Shh = sparse.diags(list(sigma_h) + sigma_c)**2
+    c_indices = [k for k, s in enumerate(sigma_c) if s is not None and np.isfinite(s)] if sigma_c is not None else []
+    sigma_c =   [s for    s in           sigma_c  if s is not None and np.isfinite(s)] if sigma_c is not None else []
+    i_indices = [k for k, s in enumerate(sigma_i) if s is not None and np.isfinite(s)] if sigma_i is not None else []
+    sigma_i =   [s for    s in           sigma_i  if s is not None and np.isfinite(s)] if sigma_i is not None else []
+    Shh = sparse.diags(list(sigma_h) + sigma_c + sigma_i)**2
     d = Shh.shape[0]
     r = 2 * N - 3 * I - 6 * T + d
     if r < 0:
@@ -152,6 +156,10 @@ def bacs(l, ict, Sll, Xa, Ma, P, *,
                 t = index // 6
                 R = Ma_inv[t][:3, :3].T # restrict rotated translation parameters
                 Nside[3 * I + 6 * t + 3 : 3 * I + 6 * t + 6, 7 + k] = R[index % 6 - 3]
+        for k, index in enumerate(i_indices):
+            i = index // 3
+            R = nullXa[:, :, i]
+            Nside[3 * i : 3 * i + 3, 7 + len(c_indices) + k] = R[index % 3] # restrict rotated object points
 
         # parameter and observation updates
         A = sparse.hstack((C, D))
@@ -237,7 +245,7 @@ def bacs(l, ict, Sll, Xa, Ma, P, *,
             w[btk] = s0 * k * w[btk] / res[btk, None]
             W = sparse.diags(w.repeat(2))
 
-    # correct direction of homogeneus scene points at the horizon
+    # correct direction of homogeneous scene points at the horizon
     Xa[-1, Xa[-1, :] < 0] = -Xa[-1, Xa[-1, :] < 0]
 
     # invert for motion matrix from world to mcs
